/*
 TDM - various "Three D Mice" implementations for GNU Maverik
 Copyright (C) 1999-2001  Advanced Interfaces Group

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA


 The authors can be contacted via:
 www   - http://aig.cs.man.ac.uk
 email - tdm@aig.cs.man.ac.uk
 mail  - Advanced Interfaces Group, Room 2.94, Kilburn Building, 
         University of Manchester, Manchester, M13 9PL, UK
*/


#ifndef _TDM_INCLUDE
#define _TDM_INCLUDE

#ifdef __cplusplus
extern "C" {
#endif

#define TDM_MAX_TRACKERS 10
#define TDM_MAX_BUTTONS  10 



/* Position and orientation data structure */

#define TDM_OUT_OF_RANGE 1
#define TDM_CALIBRATED 2

typedef struct {
  float xpos; /* Tracker's X position in transmitter's coordinate frame */
  float ypos; /* Ditto Y */
  float zpos; /* Ditto Z */
  float u[3]; /* Directional vector of tracker's X axis in transmitter's coordinate frame */
  float v[3]; /* Ditto for Y axis */
  float n[3]; /* Ditto for Z axis */
  int status; /* Position status */
} TDM_position;



/* Button event data structure */

typedef struct {
  int tracker; /* Tracker which generated the event */
  int button;  /* Button which generated the event */
  int movement; /* 1=pressed, 0=released opposite to Maverik and X */
  int button_status[TDM_MAX_TRACKERS][TDM_MAX_BUTTONS]; /* Status of all buttons when event occurred */
  TDM_position pos[TDM_MAX_TRACKERS]; /* Pos of all trackers when event occurred */
} TDM_buttonEvent; 



/* Prototypes for the functions the library needs to provide */

int  TDM_init(void);
void TDM_getPosition(int tracker, TDM_position *pos);
void TDM_getButtons(int button_status[TDM_MAX_TRACKERS][TDM_MAX_BUTTONS]);
int  TDM_getEvent(TDM_buttonEvent *event);

#ifdef __cplusplus
}
#endif
#endif
