/*
 TDM - various "Three D Mice" implementations for GNU Maverik
 Copyright (C) 1999-2001  Advanced Interfaces Group

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA


 The authors can be contacted via:
 www   - http://aig.cs.man.ac.uk
 email - tdm@aig.cs.man.ac.uk
 mail  - Advanced Interfaces Group, Room 2.94, Kilburn Building, 
         University of Manchester, Manchester, M13 9PL, UK
*/


#include "gloveprivate.h"
#include <math.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/time.h>
#include <unistd.h>

#include <termios.h>
#include <unistd.h>

#include "vmath.h"

int gloveBox= -1;

extern int glove_mode;
extern int glove_axis;
extern float glove_gain;


int gloveOpen(char *port)
{
  struct termios tty;
  int i,retry;
  unsigned char c[32];

  /* Open the serial port to the glove */
  gloveBox = open(port, O_RDWR);

  if (gloveBox == -1) {
    fprintf(stderr, "Could not open glove on device %s : ", port);
    perror("");
    return(0);
  }

  /* Read the present state of the port */
  if (tcgetattr(gloveBox, &tty) == -1) {
    perror("Could not read glove ioctl settings ");
    return(0);
  }

  /* Update it with our settings */
  tty.c_iflag = IGNBRK | IGNPAR;
  tty.c_oflag = 0;
  tty.c_lflag = 0;
  /*  tty.c_line = 0;*/
  tty.c_cc[VTIME] = 0;
  tty.c_cc[VMIN] = 1;

  cfsetispeed(&tty, B19200);
  cfsetospeed(&tty, B19200);

  /* Now set those settings onto the port */
  if (tcsetattr(gloveBox, TCSAFLUSH, &tty) == -1) {
    perror("Could not set glove ioctl settings ");
    return(0);
  }

  fprintf(stderr, "Opened connection to serial glove on %s\n", port);

  retry=0;

TryAgain:;
  /* (hopefully) get in sync with glove */
  fprintf(stderr, "Sending reset to glove device...");
  for (i=0; i<10; i++)
  {
    c[0]='A'; write(gloveBox, &c[0], 1);
    while (read(gloveBox, &c[0], 1) != 1);
    if (c[0] == 'U')
    {
      fprintf(stderr, " OK!\n");
      break;
    }
    else if (c[0] == 'Z')
    {
      perror("\nGlove EEPROM looks damaged - please rewrite it!");
      return(0);
    }
    else
    {
      fprintf(stderr, " %02Xh", c[0]);
      fflush(stderr);
    }
  }

  fprintf(stderr, "Requesting glove info...\n");
  c[0]='G'; write(gloveBox, &c[0], 1);

  for (i=0; i<32; i++)
    while (read(gloveBox, &c[i], 1) != 1);

  if ((c[0] != 'B') || (c[1] != 'R'))
  {
    perror("Got bad info header");
    return(0);
  }

  fprintf(stderr, "  Firmware version: %d.%d\n", c[2], c[3]);

  if ((c[4] & 0x41) == 0x41)
  {
    perror("Got bad handness specifier");
    return(0);
  }
  else if (c[4] & 0x01)
    fprintf(stderr, "  Glove type: right handed\n");
  else if (c[4] & 0x40)
    fprintf(stderr, "  Glove type: left handed\n");
  else
  {
    perror("Driver did not report handness");
    return(0);
  }

  fprintf(stderr, "  Pitch/Roll sensor: %s\n",
    ((c[5] & 0x10) ? "available" : "unavailable"));

  fprintf(stderr, "  Glove model: %s\n\n", (char *)&c[20]);

  fprintf(stderr, "Switching to continuous mode...\n");
  c[0]='D'; write(gloveBox, &c[0], 1);

  /* try to get at least two whole data packets */

  for (i=0; i<18; i++)
    if (read(gloveBox, &c[0], 1) != 1) break;
  
  if (i < 18)
  {

    /* something went wrong, let's use a UGLY goto :-) */

    if (retry++ < 10)
    {
      fprintf(stderr, "WARNING: not getting continuous stream (%d)\n", retry);
      goto TryAgain;
    }
    else
    {
      perror("Could not set stream data mode");
      return(0);
    }
  }

  return 1;
}



void gloveLoop(int dbg)
{
  int bytes_read;
  float x, y;
  unsigned char c[8];
  int lc=0;
  struct timeval st, ed;
  int i, oldbut[5]={0,0};

  gettimeofday(&st, NULL);

  for (;;) {
	
    for (i=0; i<8; i++) c[i]= 0;
	
    do {
      bytes_read= read(gloveBox, &c[0], 1);
    } while ((bytes_read == 1) && (c[0] != 128));
      
    if (bytes_read == 1) {
	
      /* want to wait until these bytes are available */
      for (i=0; i<8; i++)
        while (read(gloveBox, &c[i], 1) != 1);

#if 0
      fprintf(stderr, "\rf = %03d %03d %03d %03d %03d   ", c[0], c[1], c[2], c[3], c[4]);
      fprintf(stderr, "a = %+03d %+03d   ", c[5], c[6]);
      fprintf(stderr, "c = %03d ", c[7]); fflush(stdout);
#endif

      for (i=0; i<5; i++)
        shared->but[i]=c[i];
      
      for (i=0; i<5; i++) {
	if (shared->but[i]!=oldbut[i]) {
	  int k;

	  for (k=0; k<TDM_MAX_TRACKERS; k++)
	    TDM_getPosition(k, &shared->event[shared->event_in].pos[k]);

	  shared->event[shared->event_in].tracker = 0;
	  shared->event[shared->event_in].button = i;
	  shared->event[shared->event_in].movement = shared->but[i];

	  TDM_getButtons(shared->event[shared->event_in].button_status);

	  shared->event_in++;
	  if (shared->event_in == MAX_NO_EVENTS) shared->event_in=0;

	  if (shared->event_in == shared->event_out) shared->event_in--;
	  if (shared->event_in == -1) shared->event_in=MAX_NO_EVENTS-1;

	  oldbut[i]=shared->but[i];
	}
      }

      y= (127-c[5])*glove_gain/4.0;
      x= (127-c[6])*glove_gain/4.0;

      shared->x= x;
      shared->y= y;

      if (glove_mode == 0)
      {
        shared->u[0]=1; shared->u[1]=0; shared->u[2]=0;
        shared->v[0]=0; shared->v[1]=1; shared->v[2]=0;
        shared->n[0]=0; shared->n[1]=0; shared->n[2]=1;
      }
 
      {
	TDM_vector xaxis= vectorSet(1,0,0);
	TDM_vector yaxis= vectorSet(0,1,0);
	TDM_vector zaxis= vectorSet(0,0,1);

        if (glove_axis)
          axisRotate(&yaxis, -x,
            (TDM_vector *)shared->u,
            (TDM_vector *)shared->v,
            (TDM_vector *)shared->n);
        else
	  axisRotate(&zaxis, x,
            (TDM_vector *)shared->u,
            (TDM_vector *)shared->v,
            (TDM_vector *)shared->n);

	axisRotate(&xaxis, y,
          (TDM_vector *)shared->u,
          (TDM_vector *)shared->v,
          (TDM_vector *)shared->n);
      }

      lc++;
      if (lc==100 && dbg) {
	gettimeofday(&ed, NULL);
	printf("dxy %f %f  cxy %f %f time %f\n", x, y, shared->x, shared->y,
          lc*1.0/((ed.tv_sec-st.tv_sec)+(ed.tv_usec-st.tv_usec)/1.0E+6));
	lc=0;
	st=ed;
      }
    }
  }
}



void gloveClose(void)
{
  char c= 'A';
  fprintf(stderr, "Closing connection with serial glove\n");
  write(gloveBox, &c, 1);
  close(gloveBox);
}

