/*
 TDM - various "Three D Mice" implementations for GNU Maverik
 Copyright (C) 1999-2001  Advanced Interfaces Group

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA


 The authors can be contacted via:
 www   - http://aig.cs.man.ac.uk
 email - tdm@aig.cs.man.ac.uk
 mail  - Advanced Interfaces Group, Room 2.94, Kilburn Building, 
         University of Manchester, Manchester, M13 9PL, UK
*/


#include "gloveprivate.h"
#include <stdio.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <signal.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#ifdef TDM_LINUX
#define GLOVE_PORT "/dev/ttyS0"
#else
#ifdef TDM_CYGWIN
#define GLOVE_PORT "/dev/COM1"
#else
#define GLOVE_PORT "/dev/ttyd1"
#endif
#endif



/* The shared memory and its ID */

glove_shared *shared;
int ShmID;

int glove_mode;
int glove_axis;
float glove_gain;



/* Routine to clean things up on exit */

void glove_atExit()
{
  /* Close comms */
  gloveClose();

  /* Clean up shared memory */
  shmctl(ShmID, IPC_RMID, NULL); 

  exit(0);
}



/* Main */

int main(int argc, char *argv[])
{
  int i, j=0, dbg=0;
  extern char *optarg;
  char cmd[80], port[32];

  glove_mode=0;
  glove_axis=0;
  glove_gain=1.0;
  strcpy(port, GLOVE_PORT);

  while ((i=getopt(argc, argv, "adg:hp:ry"))!=-1) {
    switch (i) {
    case 'a':		/* absolute mode */
      glove_mode=0;
      break;

    case 'd':		/* enable debug messages */
      dbg=1;
      break;

    case 'g':		/* tilt sensor gain */
      glove_gain= atof(optarg);
      break;

    case 'p':		/* serial device port */
      strcpy(port, optarg);
      break;

    case 'r':		/* relative mode */
      glove_mode=1;
      break;

    case 'y':		/* use hand roll for y axis */
      glove_axis=1;
      break;

    case 'h':		/* display usage */
    case '?':
      j++;
    }
  }

  if (j) {
    printf("usage: gloved [-adhry] [-p glove port] [-g gain]\n");
    exit(1);
  }

  /* Clean up shared memory */
#ifdef TDM_LINUX
  sprintf(cmd, "ipcrm shm `ipcs -m | grep -i %X | awk '{print $2}'` >/dev/null 2>&1", SHM_KEY);
  system(cmd);
#else
#ifdef TDM_CYGWIN
  FILE *f;
  sprintf(cmd, "ipcs -m | grep -i %X | awk '{print $2}' > /tmp/.tmp", SHM_KEY);
  system(cmd);
  f= fopen("/tmp/.tmp", "r");
  if (!f) {
    printf("Cant open /tmp/.tmp\n");
    exit(1);
  }
  
  if (fscanf(f, "%i", &ShmID)==1) {
    char buf[100];
    sprintf(buf, "ipcrm shm %i", ShmID);
    printf("exec [%s]\n", buf);
    system(buf);
  }
  system("rm -f /tmp/.tmp");
    
  fclose(f);
#else
  sprintf(cmd, "ipcrm -M 0x%X 2>/dev/null", SHM_KEY);
  system(cmd);
#endif
#endif

  /* Create the shared memory segment */
  ShmID= shmget(SHM_KEY, sizeof(glove_shared), (IPC_CREAT | IPC_EXCL | 0666));

  /* Attach a pointer to it */ 
  shared= (glove_shared *)shmat(ShmID,0,0);

  if ((int)shared == -1) {
    perror("Serial glove daemon: Could not attach shared memory ");
    exit(-1);
  }

  /* Initalise shared variables */
  shared->x= 50000; shared->y= 50000;
  shared->event_in= 0; shared->event_out= 0;
  shared->but[0]=0; shared->but[1]=0; shared->but[2]=0;
  shared->but[3]=0; shared->but[4]=0;
  shared->u[0]= 1; shared->u[1]= 0; shared->u[2]= 0;
  shared->v[0]= 0; shared->v[1]= 1; shared->v[2]= 0;
  shared->n[0]= 0; shared->n[1]= 0; shared->n[2]= 1;

  /* Catch signals to remove shared memory  */
  signal(SIGHUP, glove_atExit);
  signal(SIGINT, glove_atExit);
  signal(SIGTERM, glove_atExit);
  
  /* Open connect with serial port and enter an endless sampling loop */
  if (!gloveOpen(port)) glove_atExit();
  gloveLoop(dbg);

  glove_atExit();

  return 1;
}

