/*
 TDM - various "Three D Mice" implementations for GNU Maverik
 Copyright (C) 1999-2001  Advanced Interfaces Group

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA


 The authors can be contacted via:
 www   - http://aig.cs.man.ac.uk
 email - tdm@aig.cs.man.ac.uk
 mail  - Advanced Interfaces Group, Room 2.94, Kilburn Building, 
         University of Manchester, Manchester, M13 9PL, UK
*/


#include "vmath.h"

TDM_vector vectorSet(float x, float y, float z)
{
  TDM_vector rv;
  rv.x= x; rv.y= y; rv.z= z;
  return rv;
}

TDM_vector vectorMult(TDM_vector v, TDM_matrix m)
{
  TDM_vector rv;
  rv.x= v.x*m.mat[0][0]+v.y*m.mat[0][1]+v.z*m.mat[0][2]+m.mat[0][3];
  rv.y= v.x*m.mat[1][0]+v.y*m.mat[1][1]+v.z*m.mat[1][2]+m.mat[1][3];
  rv.z= v.x*m.mat[2][0]+v.y*m.mat[2][1]+v.z*m.mat[2][2]+m.mat[2][3];
  return rv;
}

TDM_quaternion quaternionSet(TDM_vector ax, float ang)
{
  TDM_quaternion q;
  float sa, ca;
  ang*=TDM_PI_OVER_180;
  sa= sin(ang*0.5); ca= cos(ang*0.5);
  q.x= ax.x*sa; q.y= ax.y*sa; q.z= ax.z*sa; q.w= ca;
  return q;
}

TDM_matrix matrixSet(
  float roll, float pitch, float yaw, float xpos, float ypos, float zpos)
{
  TDM_matrix m;
  float cosroll,sinroll,cospitch,sinpitch,cosyaw,sinyaw;
  float sinyawcosroll,sinyawsinroll;

  roll*=TDM_PI_OVER_180; pitch*=TDM_PI_OVER_180; yaw*=TDM_PI_OVER_180;

  cosroll=cos(roll); sinroll=sin(roll);
  cospitch=cos(pitch); sinpitch=sin(pitch);
  cosyaw=cos(yaw); sinyaw=sin(yaw);

  sinyawcosroll=sinyaw*cosroll;
  sinyawsinroll=sinyaw*sinroll;

  m.mat[0][0]=cosyaw*cosroll;
  m.mat[0][1]=sinpitch*sinyawcosroll - cospitch*sinroll;
  m.mat[0][2]=cospitch*sinyawcosroll + sinpitch*sinroll;
  m.mat[0][3]=xpos;

  m.mat[1][0]=cosyaw*sinroll;
  m.mat[1][1]=sinpitch*sinyawsinroll + cospitch*cosroll;
  m.mat[1][2]=cospitch*sinyawsinroll - sinpitch*cosroll;
  m.mat[1][3]=ypos;

  m.mat[2][0]=-sinyaw;
  m.mat[2][1]=cosyaw*sinpitch;
  m.mat[2][2]=cosyaw*cospitch;
  m.mat[2][3]=zpos;

  m.mat[3][0]=0.0;
  m.mat[3][1]=0.0;
  m.mat[3][2]=0.0;
  m.mat[3][3]=1.0;

  return m;
}

TDM_matrix matrixQuaternionConvert(TDM_quaternion q)
{
  TDM_matrix m;
  m.mat[0][0]=1.0-2.0*(q.y*q.y+q.z*q.z); m.mat[0][1]=2.0*(q.x*q.y-q.w*q.z);
  m.mat[0][2]=2.0*(q.x*q.z+q.w*q.y); m.mat[0][3]=0.0;
  m.mat[1][0]=2.0*(q.x*q.y+q.w*q.z); m.mat[1][1]=1.0-2.0*(q.x*q.x+q.z*q.z);
  m.mat[1][2]=2.0*(q.y*q.z-q.w*q.x); m.mat[1][3]=0.0;
  m.mat[2][0]=2.0*(q.x*q.z-q.w*q.y); m.mat[2][1]=2.0*(q.y*q.z+q.w*q.x);
  m.mat[2][2]=1.0-2.0*(q.x*q.x+q.y*q.y); m.mat[2][3]=0.0;
  m.mat[3][0]=0.0; m.mat[3][1]=0.0; m.mat[3][2]=0.0; m.mat[3][3]=1.0;
  return m;
}

void axisRotate(TDM_vector *axis, float amt,
  TDM_vector *v1, TDM_vector *v2, TDM_vector *v3)
{
  TDM_matrix m;
  m= matrixQuaternionConvert(quaternionSet(*axis, amt));
  *v1=vectorMult(*v1,m); *v2=vectorMult(*v2,m); *v3=vectorMult(*v3,m);
}
