/*
 TDM - various "Three D Mice" implementations for GNU Maverik
 Copyright (C) 1999-2001  Advanced Interfaces Group

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA


 The authors can be contacted via:
 www   - http://aig.cs.man.ac.uk
 email - tdm@aig.cs.man.ac.uk
 mail  - Advanced Interfaces Group, Room 2.94, Kilburn Building, 
         University of Manchester, Manchester, M13 9PL, UK
*/


#include "fobprivate.h"
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <unistd.h>
#include <stdio.h>
#include "maverik.h"



/* Shared memory structure */

FOB_shared *shared;



/* Routine to initialise TDM */

int TDM_init(void)
{
  int ShmID;

  /* Get the shared memory segment (created by the daemon) */
  ShmID = shmget(SHM_KEY, sizeof(FOB_shared), 0666);
  if (ShmID == -1) return 0;

  /* Attach a pointer to it */
  shared = (FOB_shared *) shmat(ShmID,0,0);
  if ((int)shared == -1) return 0;

  /* Check memeory still up after 1 sec */
  sleep(1);
  ShmID = shmget(SHM_KEY, sizeof(FOB_shared), 0666);
  if (ShmID == -1) return 0;

  return 1;
} 



/* Routine to get button status - use dummy data */

void TDM_getButtons(int button_status[TDM_MAX_TRACKERS][TDM_MAX_BUTTONS])
{
  int i, j;

  for (i=0; i<TDM_MAX_TRACKERS; i++) {
    for (j=0; j<TDM_MAX_BUTTONS; j++) {
      button_status[i][j]= 0;
    }
  }
}



/* Routine to get the position and orientation of a tracker */

void TDM_getPosition(int tracker, TDM_position *pos)
{
  MAV_quaternion q;
  MAV_matrix m;

  pos->xpos= shared->x[tracker];
  pos->ypos= shared->y[tracker];
  pos->zpos= shared->z[tracker];

  q.w= shared->q0[tracker];
  q.x= shared->q1[tracker];
  q.y= shared->q2[tracker];
  q.z= shared->q3[tracker];
  m= mav_matrixQuaternionConvert(q);

  pos->u[0]= m.mat[MAV_MATRIX_XAXIS_X];
  pos->u[1]= m.mat[MAV_MATRIX_XAXIS_Y];
  pos->u[2]= m.mat[MAV_MATRIX_XAXIS_Z];

  pos->v[0]= m.mat[MAV_MATRIX_YAXIS_X];
  pos->v[1]= m.mat[MAV_MATRIX_YAXIS_Y];
  pos->v[2]= m.mat[MAV_MATRIX_YAXIS_Z];

  pos->n[0]= m.mat[MAV_MATRIX_ZAXIS_X];
  pos->n[1]= m.mat[MAV_MATRIX_ZAXIS_Y];
  pos->n[2]= m.mat[MAV_MATRIX_ZAXIS_Z];
}



/* Routine to get the next event */

int TDM_getEvent(TDM_buttonEvent *event)
{
  /* Return zero for no event */
  return 0;
}
