/*
 TDM - various "Three D Mice" implementations for GNU Maverik
 Copyright (C) 1999-2001  Advanced Interfaces Group

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA


 The authors can be contacted via:
 www   - http://aig.cs.man.ac.uk
 email - tdm@aig.cs.man.ac.uk
 mail  - Advanced Interfaces Group, Room 2.94, Kilburn Building, 
         University of Manchester, Manchester, M13 9PL, UK
*/


#include "fobprivate.h"
#include <stdio.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <signal.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#ifdef TDM_LINUX
#define FOB_BOX "/dev/cua0"
#else
#ifdef __CYGWIN__
#define FOB_BOX "/dev/COM1"
#else
#define FOB_BOX "/dev/ttyd2"
#endif
#endif



/* The shared memory and its ID */

FOB_shared *shared;
int ShmID;



/* Routine to clean things up on exit */

void fobAtExit()
{
  /* Close comms */
  fobClose();

  /* Clean up shared memory */
  shmctl(ShmID, IPC_RMID, NULL); 

  exit(0);
}



/* Main */

int main(int argc, char *argv[])
{
  int i, j=0, dbg=0;
  extern char *optarg;
  char port[256];

  strcpy(port, FOB_BOX);

  while ((i=getopt(argc, argv, "dhp:"))!=-1) {
    switch (i) {
    case 'd':
      dbg=1;
      break;

    case 'p':
      strcpy(port, optarg);
      break;

    case 'h':
    case '?':
      j++;
    }
  }

  if (j) {
    printf("usage: %s [-d] [-p fob port]\n", argv[0]);
    exit(1);
  }

  /* Clean up any shared memory hanging about */
#ifdef TDM_LINUX
  {
    FILE *f;
    system("ipcs -m | grep -i 503 | awk '{print $2}' > /tmp/.tmp");
    f= fopen("/tmp/.tmp", "r");
    if (!f) {
      printf("Cant open /tmp/.tmp\n");
      exit(1);
    }
  
    if (fscanf(f, "%i", &ShmID)==1) {
      char buf[100];
      sprintf(buf, "ipcrm shm %i", ShmID);
      system(buf);
    }
    system("rm -f /tmp/.tmp");
    
    fclose(f);
  }
#else
  system("ipcrm -M 0x503 2>/dev/null"); /* 503= SHM_KEY in hex */
#endif

  /* Create the shared memory segment */
  ShmID = shmget(SHM_KEY, sizeof(FOB_shared), (IPC_CREAT | IPC_EXCL | 0666));

  /* Attach a pointer to it */ 
  shared = (FOB_shared *) shmat(ShmID,0,0);

  if ((int)shared == -1) {
    perror("FOB server: Could not attach shared memory ");
    exit(-1);
  }

  /* Initalise shared variables */
  for (i=0; i<TDM_MAX_TRACKERS; i++) {
    shared->x[i]=0;
    shared->y[i]=0;
    shared->z[i]=20;
    shared->q0[i]=1;
    shared->q1[i]=0;
    shared->q2[i]=0;
    shared->q3[i]=0;
  }

  /* Catch signals to remove shared memory */
  signal(SIGHUP, fobAtExit);
  signal(SIGINT, fobAtExit);
  signal(SIGTERM, fobAtExit);

  /* Open connect to box and enter an endless sampling loop */
  if (!fobOpen(port)) fobAtExit();
  fobLoop(dbg);

  fobAtExit();

  return 1;
}
