/*
 TDM - various "Three D Mice" implementations for GNU Maverik
 Copyright (C) 1999-2001  Advanced Interfaces Group

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA


 The authors can be contacted via:
 www   - http://aig.cs.man.ac.uk
 email - tdm@aig.cs.man.ac.uk
 mail  - Advanced Interfaces Group, Room 2.94, Kilburn Building, 
         University of Manchester, Manchester, M13 9PL, UK
*/


#include "it30private.h"
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <unistd.h>
#include <stdio.h>
#include "maverik.h"



/* Shared memory structure */

IT30_shared *shared;



/* Routine to initialise TDM */

int TDM_init(void)
{
  int ShmID;

  /* Get the shared memory segment (created by the daemon) */
  ShmID = shmget(SHM_KEY, sizeof(IT30_shared), 0666);
  if (ShmID == -1) return 0;

  /* Attach a pointer to it */
  shared = (IT30_shared *) shmat(ShmID,0,0);
  if ((int)shared == -1) return 0;

  /* Check memeory still up after 1 sec */
  sleep(1);
  ShmID = shmget(SHM_KEY, sizeof(IT30_shared), 0666);
  if (ShmID == -1) return 0;

  return 1;
} 



/* Routine to get button status - use dummy data */

void TDM_getButtons(int button_status[TDM_MAX_TRACKERS][TDM_MAX_BUTTONS])
{
  int i, j;

  for (i=0; i<TDM_MAX_TRACKERS; i++) {
    for (j=0; j<TDM_MAX_BUTTONS; j++) {
      button_status[i][j]= 0;
    }
  }
}



/* Routine to get the position and orientation of a tracker */

void TDM_getPosition(int tracker, TDM_position *pos)
{
  MAV_matrix m;
  float roll, pitch, yaw;

  if (tracker==0)
  {
    /* Calculate angles */
    roll= (shared->roll[0]/65536.0)*360.0;
    pitch= (shared->pitch[0]/65536.0)*360.0;
    yaw= (shared->yaw[0]/65536.0)*360.0;

    /* Form a matrix from them */
    m= mav_matrixSet(roll, pitch, yaw, 0, 0, 0);

    /* Store orientation as direction vectors */
    pos->u[0]=m.mat[0][0];
    pos->u[1]=m.mat[1][0];
    pos->u[2]=m.mat[2][0];
    pos->v[0]=m.mat[0][1];
    pos->v[1]=m.mat[1][1];
    pos->v[2]=m.mat[2][1];
    pos->n[0]=m.mat[0][2];
    pos->n[1]=m.mat[1][2];
    pos->n[2]=m.mat[2][2];

    /* Dummy position */
    pos->xpos= 0;
    pos->ypos= 0;
    pos->zpos= 20;
  }
  else
  {
    /* Dummy data for other trackers */
    pos->xpos= 0;
    pos->ypos= 0;
    pos->zpos= 20;
    pos->u[0]=1;
    pos->u[1]=0;
    pos->u[2]=0;
    pos->v[0]=0;
    pos->v[1]=1;
    pos->v[2]=0;
    pos->n[0]=0;
    pos->n[1]=0;
    pos->n[2]=1;
  }

  pos->status= 0;
}



/* Routine to get the next event */

int TDM_getEvent(TDM_buttonEvent *event)
{
  /* Return zero for no event */
  return 0;
}
