/*
 TDM - various "Three D Mice" implementations for GNU Maverik
 Copyright (C) 1999-2001  Advanced Interfaces Group

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA


 The authors can be contacted via:
 www   - http://aig.cs.man.ac.uk
 email - tdm@aig.cs.man.ac.uk
 mail  - Advanced Interfaces Group, Room 2.94, Kilburn Building, 
         University of Manchester, Manchester, M13 9PL, UK
*/


#include "magprivate.h"
#include <stdio.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <signal.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#ifdef TDM_LINUX
#define MAGELLAN_BOX1 "/dev/cua0"
#define MAGELLAN_BOX2 "/dev/cua1"
#else
#ifdef __CYGWIN__
#define MAGELLAN_BOX1 "/dev/COM1"
#define MAGELLAN_BOX2 "/dev/COM2"
#else
#define MAGELLAN_BOX1 "/dev/ttyd1"
#define MAGELLAN_BOX2 "/dev/ttyd2"
#endif
#endif



/* The shared memory and its ID */

Mag_shared *shared;
int ShmID;



/* Routine to clean things up on exit */

void Mag_atExit()
{
  /* Close comms */
  magellanClose();

  /* Clean up shared memory */
  shmctl(ShmID, IPC_RMID, NULL); 

  exit(0);
}



/* Main */

int main(int argc, char *argv[])
{
  int i, j=0, dbg=0;
  extern char *optarg;  
  char port[2][256];
  int nmag=1;
  int pid=1;
  int magnum;

  strcpy(port[0], MAGELLAN_BOX1);
  strcpy(port[1], MAGELLAN_BOX2);

  while ((i=getopt(argc, argv, "dhm:p:q:")) != -1 ) {
    switch (i) {
    case 'd':
      dbg=1;
      break;

    case 'p':
      strcpy(port[0], optarg);
      break;

    case 'q':
      strcpy(port[1], optarg);
      nmag=2;
      break;

    case 'm':
      nmag= atof(optarg);
      if (nmag!=1 && nmag!=2) j++;
      break;

    case 'h':
    case '?':
      j++;
    }
  }

  if (j) {
    printf("usage: %s [-d] [-m 1 or 2] [-p magellan port 1] [-q magellan port 2]\n", argv[0]);
    exit(1); 
  }

  /* Fork if we have more than one Magellan */
  if (nmag==2) {
    pid= fork();

    if (pid==-1) {
      perror("fork failed");
      exit(1);
    }
  }

  if (pid!=0) 
  {
    /* Parent cleans up shared memory */
#ifdef TDM_LINUX
    FILE *f;
    system("ipcs -m | grep -i 506 | awk '{print $2}' > /tmp/.tmp");
    f= fopen("/tmp/.tmp", "r");
    if (!f) {
      printf("Cant open /tmp/.tmp\n");
      exit(1);
    }
  
    if (fscanf(f, "%i", &ShmID)==1) {
      char buf[100];
      sprintf(buf, "ipcrm shm %i", ShmID);
      printf("exec [%s]\n", buf);
      system(buf);
    }
    system("rm -f /tmp/.tmp");
    
    fclose(f);
#else
    system("ipcrm -M 0x506 2>/dev/null"); /* 506= SHM_KEY in hex */
#endif

    /* Parent create the shared memory segment */
    ShmID = shmget(SHM_KEY, sizeof(Mag_shared), (IPC_CREAT | IPC_EXCL | 0666));

    magnum=0;
  }
  else
  {
    /* Child waits for parent to create shared mem, then gets it */
    sleep(5);
    ShmID = shmget(SHM_KEY, sizeof(Mag_shared), 0666);

    magnum=1;
  }

  /* Attach a pointer to it */ 
  shared = (Mag_shared *) shmat(ShmID,0,0);

  if ((int)shared == -1) {
    perror("Magellan server: Could not attach shared memory ");
    exit(-1);
  }

  /* Parent initalise shared variables */
  if (pid!=0) {
    shared->event_in=0;
    shared->event_out=0;
    for (i=0; i<TDM_MAX_TRACKERS; i++) {
      shared->pos[i].xpos=0;
      shared->pos[i].ypos=0;
      shared->pos[i].zpos=20;
      shared->pos[i].u[0]=1;
      shared->pos[i].u[1]=0;
      shared->pos[i].u[2]=0;
      shared->pos[i].v[0]=0;
      shared->pos[i].v[1]=1;
      shared->pos[i].v[2]=0;
      shared->pos[i].n[0]=0;
      shared->pos[i].n[1]=0;
      shared->pos[i].n[2]=1;
      for (j=0; j<TDM_MAX_BUTTONS; j++) shared->new_status[i][j]= 0;
    }
  }

  /* Catch signals to remove shared memory  */
  signal(SIGHUP, Mag_atExit);
  signal(SIGINT, Mag_atExit);
  signal(SIGTERM, Mag_atExit);

  /* Open connect with box and enter an endless sampling loop */
  if (!magellanOpen(port[magnum])) Mag_atExit();
  magellanLoop(dbg, magnum);

  Mag_atExit();

  return 1;
}
