/*
 TDM - various "Three D Mice" implementations for GNU Maverik
 Copyright (C) 1999-2001  Advanced Interfaces Group

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA


 The authors can be contacted via:
 www   - http://aig.cs.man.ac.uk
 email - tdm@aig.cs.man.ac.uk
 mail  - Advanced Interfaces Group, Room 2.94, Kilburn Building, 
         University of Manchester, Manchester, M13 9PL, UK
*/


#define _OLD_TERMIOS
#include "magprivate.h"
#include "maverik.h"
#include <termio.h>
#include <termios.h>
#include <math.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>

int magellanBox= -1;



void magellanWrite(char *buf)
{
  int rv;
  int num= strlen(buf);

  /* Write the buffer and check its all gone */
  rv= write(magellanBox, buf, num);
  if (rv!=num) printf("rv!=num\n");
}



void magellanRead(int num, char *buf)
{
  fd_set pf;
  int rv, totread=0;

  do {
    FD_ZERO(&pf);
    FD_SET(magellanBox, &pf);
    select(200, &pf, NULL, NULL, NULL);
    rv= read(magellanBox, &buf[totread], num-totread);
    totread+=rv;
  } while (totread!=num);
  
  buf[num]=0;
}



void magellanReset(void)
{
  char res[500];

  /* Reset */
  fprintf(stderr, "Requesting reset... ");
  magellanWrite("\rz\r");
  fprintf(stderr, "done. Reading response... ");
  magellanRead(2, res);
  fprintf(stderr, "done\n");
  if (strcmp(res, "z\r")) {
    fprintf(stderr, "Failed to reset device\n");
    fprintf(stderr, "Read [%s] when expecting [%s]\n", res, "z\r");
    exit(1);
  }
}



void magellanInit(void)
{
  char res[500];
  int count=0;

  /* Reset */
  magellanReset();
  sleep(1);
  magellanReset();

  /* Get version */
  fprintf(stderr, "Requesting version... ");
  magellanWrite("vQ\r");
  fprintf(stderr, "done. Reading response... ");
  do {
    magellanRead(1, &res[count]);
    count+=1;
  } while (res[count-1]!=13);  

  fprintf(stderr, "done\nVersion: %s\n", res);
}



int magellanOpen(char *name)
{
  struct termio mysettings;
  
  /* Open the serial port to the Magellan box */
  magellanBox = open(name, O_RDWR);

  if (magellanBox == -1) {
    fprintf(stderr, "Could not open Magellan box on device %s : ", name);
    perror("");
    return(0);
  }

  /* Read the present state of the port */
  if (ioctl(magellanBox, TCGETA, &mysettings ) == -1) {
    perror("Could not read Magellan ioctl settings ");
    return(0);
  }

  /* Update it with our settings */
  mysettings.c_iflag= IGNBRK|IGNPAR;
  mysettings.c_oflag = 0;
  mysettings.c_cflag = B9600 | CREAD | CS8 | CSTOPB | HUPCL; /* | CLOCAL;*/
  mysettings.c_lflag = 0;				/* -ECHO */
#ifdef TDM_IRIX6
  mysettings.c_line  = LDISC1;
#endif
  mysettings.c_cc[VEOL] = '\r';
  mysettings.c_cc[VERASE] = '\000';
  mysettings.c_cc[VKILL] = '\000';
  mysettings.c_cc[VMIN ] = 1;
  mysettings.c_cc[VTIME] = 0;

  /* Now set those settings onto the port */
  if (ioctl(magellanBox, TCSETA, &mysettings) ==-1) {
    perror("Could not set Magellan ioctl settings ");
    return(0);
  }

  sleep(1);
  
  /* Initialise the device */
  magellanInit();

  return 1;
}



int conval(char c)
{
  int rv;

  /* Character to nibble code conversion */
  switch (c) {
  case '0':
    rv=0;
    break;
  case 'A':
    rv=1;
    break;
  case 'B':
    rv=2;
    break;
  case '3':
    rv=3;
    break;
  case 'D':
    rv=4;
    break;
  case '5':
    rv=5;
    break;
  case '6':
    rv=6;
    break;
  case 'G':
    rv=7;
    break;
  case 'H':
    rv=8;
    break;
  case '9':
    rv=9;
    break;
  case ':':
    rv=10;
    break;
  case 'K':
    rv=11;
    break;
  case '<':
    rv=12;
    break;
  case 'M':
    rv=13;
    break;
  case 'N':
    rv=14;
    break;
  case '?':
    rv=15;
    break;
  default:
    fprintf(stderr, "Failed to convert character %c\n", c);
    rv=0;
    break;
  }

  return rv;
}

int conv(char *buf)
{
  /* Calculate value of 4 nibble  */
  return conval(buf[0])*4096+conval(buf[1])*256+conval(buf[2])*16+conval(buf[3])-32768;
}



MAV_matrix mav_matrixSet(float roll, float pitch, float yaw, float xpos, float ypos, float zpos)
{
  MAV_matrix mat;
  float cosroll,sinroll,cospitch,sinpitch,cosyaw,sinyaw;
  float sinyawcosroll,sinyawsinroll;
  
/* Convert to radians */

  roll*=MAV_PI_OVER_180;
  pitch*=MAV_PI_OVER_180;
  yaw*=MAV_PI_OVER_180;

/* Calculate trig functions */
  cosroll=cos(roll);
  sinroll=sin(roll);
  cospitch=cos(pitch);
  sinpitch=sin(pitch);
  cosyaw=cos(yaw);
  sinyaw=sin(yaw);
  sinyawcosroll=sinyaw*cosroll;
  sinyawsinroll=sinyaw*sinroll;
  
/* Form the matrix */

  mat.mat[0][0] = cosyaw*cosroll;
  mat.mat[0][1] = sinpitch*sinyawcosroll - cospitch*sinroll;
  mat.mat[0][2] = cospitch*sinyawcosroll + sinpitch*sinroll;
  mat.mat[0][3] = xpos;

  mat.mat[1][0] = cosyaw*sinroll;
  mat.mat[1][1] = sinpitch*sinyawsinroll + cospitch*cosroll;
  mat.mat[1][2] = cospitch*sinyawsinroll - sinpitch*cosroll;
  mat.mat[1][3] = ypos;

  mat.mat[2][0] = -sinyaw;
  mat.mat[2][1] = cosyaw*sinpitch;
  mat.mat[2][2] = cosyaw*cospitch;
  mat.mat[2][3] = zpos;

  mat.mat[3][0] = 0.0;
  mat.mat[3][1] = 0.0;
  mat.mat[3][2] = 0.0;
  mat.mat[3][3] = 1.0;

  return mat;
}



void magellanLoop(int dbg, int magnum)
{
  MAV_matrix m;
  char buf[500];
  int count;
  int i, j, k, old_status[TDM_MAX_TRACKERS][TDM_MAX_BUTTONS], firsttime=1, keyv;

  /* Request position and orientation */
  magellanWrite("m3\r");

  /* Request key status */
  magellanWrite("kQ\r");

  while (1) {
    
    /* Read first character of packet - this identifies it */
    magellanRead(1, buf);
    switch (buf[0]) {

    case 'd': /* Position/orientation packet */
      magellanRead(25, &buf[1]);

      /* Calc position */
      shared->pos[magnum].xpos= conv(&buf[1])*0.1;
      shared->pos[magnum].ypos= conv(&buf[5])*0.1;
      shared->pos[magnum].zpos= 20+conv(&buf[9])*0.1;
      
      /* Calc orientation */
      m= mav_matrixSet(conv(&buf[21])*0.5, conv(&buf[13])*0.5, conv(&buf[17])*0.5, 0, 0, 0);
      shared->pos[magnum].u[0]=m.mat[0][0];
      shared->pos[magnum].u[1]=m.mat[1][0];
      shared->pos[magnum].u[2]=m.mat[2][0];
      shared->pos[magnum].v[0]=m.mat[0][1];
      shared->pos[magnum].v[1]=m.mat[1][1];
      shared->pos[magnum].v[2]=m.mat[2][1];
      shared->pos[magnum].n[0]=m.mat[0][2];
      shared->pos[magnum].n[1]=m.mat[1][2];
      shared->pos[magnum].n[2]=m.mat[2][2];

      shared->pos[magnum].status= 0;
      break;

    case 'k': /* Key event packet */
      magellanRead(4, &buf[1]);

      /* Calculate key value using a bit for each key  */
      keyv= conval(buf[1])+conval(buf[2])*16+conval(buf[3])*256;

      /* 
	 The least significant 9 bits of the of data represent the keys 1-9 (9=*) 
	 with pressed equals 1 and released equals 0. 

	 Store the new status of each button in an array by right shifting through
	 the data and logically adding it with 1 to yield 0 or 1 for each button.
      */

      for (i=0; i<9; i++) shared->new_status[magnum][i+1] = ((keyv >> i) & 1);

      /* 
	 For the first time through the loop case set the old status of the 
	 buttons to the new status so we have something to check against.
      */

      if (firsttime) {
	firsttime = 0;
	for (i=0; i<TDM_MAX_TRACKERS; i++) {
	  for (j=0; j<TDM_MAX_BUTTONS; j++) {
	    old_status[i][j] = shared->new_status[i][j];
	  }
	}
      }

      /*
	Loop over all buttons of all trackers. Check to see if the new status this
	time around the loop is different to the status last time around the 
	loop. If it is then the button was either pressed or released, the
	action is given by the new status.
      */
    
      for (i=0; i<TDM_MAX_TRACKERS; i++) {
	for (j=0; j<TDM_MAX_BUTTONS; j++) {
	
	  if (shared->new_status[i][j] != old_status[i][j]) {
	  
	    /* Update event queue - All data is placed in the event_in index of the queue array */
	  
	    /*
	      Copy the latest position and orientation of each active tracker so as we
	      know this information the (more-or-less) instance the event occurred. 
	    */

	    for (k=0; k<TDM_MAX_TRACKERS; k++) TDM_getPosition(k, &shared->event[shared->event_in].pos[k]);
	    
	    /* 
	       Copy which tracker, button and movement caused the event and the status
	       of all other buttons. 
	    */

	    shared->event[shared->event_in].tracker = i;
	    shared->event[shared->event_in].button = j;
	    shared->event[shared->event_in].movement = shared->new_status[i][j];
	  
	    TDM_getButtons(shared->event[shared->event_in].button_status);
	  
	    /* Update the pointer to the position where the next event is to be stored */
	    shared->event_in++;
	    if (shared->event_in == MAX_NO_EVENTS) shared->event_in=0;
	  
	    /* 
	       If the new event in position is the same as the event out position then there is no room
	       for the next event. In this case decrease the event in pointer by one. This has the effect
	       that the next event will overwrite the most resent event. This is good since we will always
	       have the last event, and each event carries the state of all buttons.
	     */
	  
	    if (shared->event_in == shared->event_out) shared->event_in--;
	    if (shared->event_in == -1) shared->event_in=MAX_NO_EVENTS-1;
	  }
	
	  /*
	    Update the old status of the mouse to the new status ready for the next
	    time around the loop.
	  */

	  old_status[i][j] = shared->new_status[i][j];
	}
      }

      break;

    case 'm': /* Mode change */
      magellanRead(2, &buf[1]);
      break;

    case 'z': /* Reset */
      magellanRead(1, &buf[1]);
      break;

    case 'n': /* Zero (null) radius */
      magellanRead(2, &buf[1]);
      break;

    case 'q': /* Sensitivity */
      magellanRead(3, &buf[1]);
      break;

    default: /* Unknown packet - read until end of line */
      count=1;
      do {
	magellanRead(1, &buf[count]);
	count++;
      } while (buf[count-1]!=13);  

      fprintf(stderr, "Unknown packet [%s]\n", buf);

      break;
    }

    if (dbg==1) {
      /* Replace \r with NULL termination then output it */
      buf[strlen(buf)-1]=0;
      printf("Packet [%i] - [%s]\n", magnum, buf);
    }
  }
}



void magellanClose()
{
  fprintf(stderr, "Closing connection to Magellan\n");
  close(magellanBox);
}

