/*
 TDM - various "Three D Mice" implementations for GNU Maverik
 Copyright (C) 1999-2001  Advanced Interfaces Group

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA


 The authors can be contacted via:
 www   - http://aig.cs.man.ac.uk
 email - tdm@aig.cs.man.ac.uk
 mail  - Advanced Interfaces Group, Room 2.94, Kilburn Building, 
         University of Manchester, Manchester, M13 9PL, UK
*/


#include "polprivate.h"
#include <unistd.h>
#include <stdio.h>
#include <fcntl.h>
#include <termios.h>

int divisionBox= -1;


#include <unistd.h>
#include <sys/types.h>
#include <sys/time.h>
#include <stdio.h>
#include <termios.h>
#include <sys/stat.h>
#include <fcntl.h>

/* Open a connection with the Division box */

int divisionOpen(char *name)
{
  struct termios mysettings;
  unsigned char ignored;

  /* Open the serial port to the Division box */
  divisionBox = open(name, O_RDWR);

  if (divisionBox == -1) {
    fprintf(stderr, "Could not open Division box on device %s : ", name);
    perror("");
    return(0);
  }

  /* Read the present state of the port */
  if (tcgetattr(divisionBox, &mysettings) == -1) {
    perror("Could not read Division ioctl settings ");
    return(0);
  }

  /* Update it with our settings */
  mysettings.c_iflag= IGNBRK|IGNPAR;
  mysettings.c_cc[CEOF]= 1;
  mysettings.c_lflag= 0;
  cfsetospeed(&mysettings, B9600);

  /* Now set those settings onto the port */
  if (tcsetattr(divisionBox, TCSANOW, &mysettings) == -1) {
    perror("Could not set Division ioctl settings ");
    return(0);
  }

/*
   Read from port until we receive data which has bits 7 and 8 set. This 
   marks the end of a block of useful data (detailed below) and thus
   sync's the comms.
*/

  printf("Sync with Division box...\n");
  do {
    read(divisionBox, &ignored, 1);
  } while ((ignored >> 6) != 3);
  printf("done\n");

  return 1;
}



/* Endlessly read the Division box */

void divisionLoop(int debug)
{
  int i, j, k, old_status[TDM_MAX_TRACKERS][TDM_MAX_BUTTONS], firsttime=1, lc=0;
  unsigned char red_data, blue_data, ignored;

  while (1) {
    lc++;

    /*
      Read data from Divsion box. Data comes in 4 separate chunks before repeating. 
      First is data for red mouse (tracker 0), then blue mouse (tracker 1) followed 
      by 2 unused devices which are ignored.
    
      Bits 7 and 8 of the each piece of data indicate which of the 4 devices 
      (the two mice and two unused devices) the data belongs to. Make sure we
      are reading the data into the right place as it appears that the comms
      can sometime un-sync itself.
    */

    do {
      read(divisionBox, &red_data, 1);
    } while ((red_data >> 6) != 0);
    
    do {
      read(divisionBox, &blue_data, 1);
    } while ((blue_data >> 6) != 1);

    do {
      read(divisionBox, &ignored, 1);
    } while ((ignored >> 6) != 2);

    do {
      read(divisionBox, &ignored, 1);
    } while ((ignored >> 6) != 3);

    /* Print out record if debugging set */
    if (lc==100 && debug) {
      printf("rbi %i %i %i\n", red_data, blue_data, ignored);
      lc=0;
    }

    /* 
       The least significant 5 bits of the each piece of data
       represent the middle, top, right, left and bottom buttons for that 
       mouse repectivily, with 0 being pressed and 1 being released. Need to
       swap values so that pressed equals 1 and released equals 0. 

       Store the new status of each button in an array by right shifting through
       the read data and adding it with 1 to yield 0 or 1 for each button of
       each mouse.
    */

    for (i=0; i<5; i++) {
      shared->new_status[0][i] = !((red_data >> i) & 1);  /* red data corresponds to tracker 0 */
      shared->new_status[1][i] = !((blue_data >> i) & 1); /* blue data corresponds to tracker 1 */
    }

    /* 
       For the first time through the loop case set the old status of the 
       buttons to the new status so we have something to check against.
    */

    if (firsttime) {
      firsttime = 0;
      for (i=0; i<TDM_MAX_TRACKERS; i++) {
	for (j=0; j<TDM_MAX_BUTTONS; j++) {
	  old_status[i][j] = shared->new_status[i][j];
	}
      }
    }

    /*
      Loop over all buttons of all trackers. Check to see if the new status this
      time around the loop is different to the status last time around the 
      loop. If it is then the button was either pressed or released, the
      action is given by the new status.
    */
    
    for (i=0; i<TDM_MAX_TRACKERS; i++) {
      for (j=0; j<TDM_MAX_BUTTONS; j++) {
	
	if (shared->new_status[i][j] != old_status[i][j]) {
	  
	  /* Update event queue - All data is placed in the event_in index of the queue array*/
	  
	  /*
	    Copy the latest position and orientation of each active tracker so as we
	    know this information the (more-or-less) instance the event occurred. 
	  */

	  for (k=0; k<TDM_MAX_TRACKERS; k++) TDM_getPosition(k, &shared->event[shared->event_in].pos[k]);
	    
	  /* 
	     Copy which mouse, button and movement caused the event and the status
	     of all other buttons. 
	  */

	  shared->event[shared->event_in].tracker = i;
	  shared->event[shared->event_in].button = j;
	  shared->event[shared->event_in].movement = shared->new_status[i][j];
	  
	  TDM_getButtons(shared->event[shared->event_in].button_status);
	  
	  /* Update the pointer to the position where the next event is to be stored */
	  shared->event_in++;
	  if (shared->event_in == MAX_NO_EVENTS) shared->event_in=0;
	  
	  /* 
	     If the new event in position is the same as the event out position then there is no room
	     for the next event. In this case decrease the event in pointer by one. This has the effect
	     that the next event will overwrite the most resent event. This is good since we will always
	     have the last event, and each event carries the state of all buttons.
	  */
	  
	  if (shared->event_in == shared->event_out) shared->event_in--;
	  if (shared->event_in == -1) shared->event_in=MAX_NO_EVENTS-1;
	}
	
	/*
	  Update the old status of the mouse to the new status ready for the next
	  time around the loop.
	*/

	old_status[i][j] = shared->new_status[i][j];
      }
    }
  }
}



/* Routine to close the Division box */

void divisionClose(void)
{
  printf("Closing Division box\n");

  close(divisionBox);
}
