/*
 TDM - various "Three D Mice" implementations for GNU Maverik
 Copyright (C) 1999-2001  Advanced Interfaces Group

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA


 The authors can be contacted via:
 www   - http://aig.cs.man.ac.uk
 email - tdm@aig.cs.man.ac.uk
 mail  - Advanced Interfaces Group, Room 2.94, Kilburn Building, 
         University of Manchester, Manchester, M13 9PL, UK
*/


#include "polprivate.h"
#include <strings.h>



/*
  Routine to gets the status (i.e. pressed or released) of each button on each tracker.

  The status is simply copied from the "current" button status maintained
  in the shared memory, i.e. it is not the status when the routine was
  called but rather the status the last time the Division box was sampled, 
  but the data will only be a few milliseconds old.
*/

void TDM_getButtons(int button_status[TDM_MAX_TRACKERS][TDM_MAX_BUTTONS])
{
  int i, j;

  /* Copy the "current" button status to the destination */
  for (i=0; i<TDM_MAX_TRACKERS; i++) {
    for (j=0; j<TDM_MAX_BUTTONS; j++) {
      button_status[i][j] = shared->new_status[i][j];
    }
  }
}



/*
  Routine to get the position and orientation of a tracker. 

  The position and orientation data is simply copied from "current" 
  position and orientation maintained in the shared memory, i.e. it
  is not the position and orientation  when the routine was called 
  but rather the position and orientation the last time the Polhemus's
  were sampled, and as such the data could be some 10's of ms old.
*/

void TDM_getPosition(int tracker, TDM_position *pos)
{

  /* Copy the "current" position and orientation data to the destination.*/
  bcopy(&shared->pos[tracker], pos, sizeof(TDM_position)); 
}



/*
  This routine gets the next outstanding button event (if there is one).
  The events are maintained in the shared memory, and the most outstanding event is at
  event_out in the queue. The routine reads this event from the queue and updates the
  event_out pointer to the next outstanding event.
*/

int TDM_getEvent(TDM_buttonEvent *event)
{
  int rv=0;

  if (shared->event_in == shared->event_out) 
  {
    /* No events outstanding so return with value 0 */    
    rv= 0;
  }
  else
  {
    /* Copy the event at the event_out of queue into event structure passed */ 
    bcopy(&shared->event[shared->event_out], event, sizeof(TDM_buttonEvent)); 

    /* Update event_out pointer */
    shared->event_out++;
    if (shared->event_out == MAX_NO_EVENTS) shared->event_out=0;

    /* Return with a value of 1 */
    rv= 1;
  }
  
  return rv;
}
