/*
 TDM - various "Three D Mice" implementations for GNU Maverik
 Copyright (C) 1999-2001  Advanced Interfaces Group

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA


 The authors can be contacted via:
 www   - http://aig.cs.man.ac.uk
 email - tdm@aig.cs.man.ac.uk
 mail  - Advanced Interfaces Group, Room 2.94, Kilburn Building, 
         University of Manchester, Manchester, M13 9PL, UK
*/


#include "polprivate.h"
#include <stdio.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <signal.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#ifdef TDM_LINUX
#define POLHEMUS_BOX "/dev/cua0"
#define DIVISION_BOX "/dev/cua1"
#else
#ifdef __CYGWIN__
#define POLHEMUS_BOX "/dev/COM1"
#define DIVISION_BOX "/dev/COM2"
#else
#define POLHEMUS_BOX "/dev/ttyd1"
#define DIVISION_BOX "/dev/ttyd2"
#endif
#endif



/* The shared memory, its ID and the results of the fork */

Polhemus_shared *shared;
int ShmID;
int pid=1;

int longranger=0;
int distortion=0;


/* Routine to clean things up on exit */

void Polhemus_atExit()
{
  /* Close comms */
  if (pid!=0)
  {
    polhemusClose();
  }
  else
  {
    divisionClose();
  }

  /* Clean up shared memory */
  shmctl(ShmID, IPC_RMID, NULL); 

  exit(0);
}



/* Main */

int main(int argc, char *argv[])
{
  int i, j=0, dbg=0, poliso=0, divi=0;
  extern char *optarg;
  char port1[256], port2[256];

  strcpy(port1, POLHEMUS_BOX);
  strcpy(port2, DIVISION_BOX);

  while ((i=getopt(argc, argv, "vcdlp:q:hi:")) != -1 ) {
    switch (i) {
    case 'v':
      printf("Using Division box\n");
      divi=1;
      break;

    case 'c':
      printf("Correcting for distortions\n");
      distortion=1;
      break;

    case 'l':
      printf("Using Long Ranger settings\n");
      longranger=1;
      break;

    case 'd':
      dbg=1;
      break;

    case 'i':
      poliso= atof(optarg);
      if (poliso!=1 && poliso!=2) j++;
      break;

    case 'p':
      strcpy(port1, optarg);
      break;

    case 'q':	/* "p" might stand for "port"... what's "q"?!? :-) */
      strcpy(port2, optarg);
      divi=1;
      break;

    case 'h':
    case '?':
      j++;
    }
  }

  if (j) {
    printf("usage: %s [-v] [-l] [-c] [-d] [-p polhemus serial port] [-q division serial port] [-i 1 or 2]\n", argv[0]);
    exit(1); 
  }

  /* Fork. Child deals with Division box, parent with Polhemus */

  if (divi) {
    pid= fork();

    if (pid==-1) {
      perror("fork failed");
      exit(1);
    }
  }

  if (pid!=0) 
  {
    /* Parent cleans up shared memory */
#ifdef TDM_LINUX
    FILE *f;
    system("ipcs -m | grep -i 501 | awk '{print $2}' > /tmp/.tmp");
    f= fopen("/tmp/.tmp", "r");
    if (!f) {
      printf("Cant open /tmp/.tmp\n");
      exit(1);
    }
  
    if (fscanf(f, "%i", &ShmID)==1) {
      char buf[100];
      sprintf(buf, "ipcrm shm %i", ShmID);
      printf("exec [%s]\n", buf);
      system(buf);
    }
    system("rm -f /tmp/.tmp");
    
    fclose(f);
#else
    system("ipcrm -M 0x501 2>/dev/null"); /* 501= SHM_KEY in hex */
#endif

    /* Parent creates the shared memory segment */
    ShmID = shmget(SHM_KEY, sizeof(Polhemus_shared), (IPC_CREAT | IPC_EXCL | 0666));
  }
  else
  {
    /* Child waits for parent to create shared mem, then gets it */
    sleep(5);
    ShmID = shmget(SHM_KEY, sizeof(Polhemus_shared), 0666);
  }

  /* Attach a pointer to it */ 
  shared = (Polhemus_shared *) shmat(ShmID,0,0);

  if ((int)shared == -1) {
    perror("Polhemus server: Could not attach shared memory ");
    exit(-1);
  }

  /* Parent initalise shared variables */
  if (pid!=0) {
    shared->event_in=0;
    shared->event_out=0;
    for (i=0; i<TDM_MAX_TRACKERS; i++) {
      shared->pos[i].xpos=0;
      shared->pos[i].ypos=0;
      shared->pos[i].zpos=20;
      shared->pos[i].u[0]=1;
      shared->pos[i].u[1]=0;
      shared->pos[i].u[2]=0;
      shared->pos[i].v[0]=0;
      shared->pos[i].v[1]=1;
      shared->pos[i].v[2]=0;
      shared->pos[i].n[0]=0;
      shared->pos[i].n[1]=0;
      shared->pos[i].n[2]=1;

      for (j=0; j<TDM_MAX_BUTTONS; j++) shared->new_status[i][j]= 0;
    }
  }

  /* Catch signals to remove shared memory  */
  signal(SIGHUP, Polhemus_atExit);
  signal(SIGINT, Polhemus_atExit);
  signal(SIGTERM, Polhemus_atExit);

  /* Open connect with relevant box and enter an endless sampling loop */
  if (pid!=0)
  {
    if (!polhemusOpen(port1, poliso)) Polhemus_atExit();
    polhemusLoop(dbg);
  }
  else
  {
    if (!divisionOpen(port2)) Polhemus_atExit();
    divisionLoop(dbg);
  }

  Polhemus_atExit();

  return 1;
}
