/*
 TDM - various "Three D Mice" implementations for GNU Maverik
 Copyright (C) 1999-2001  Advanced Interfaces Group

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA


 The authors can be contacted via:
 www   - http://aig.cs.man.ac.uk
 email - tdm@aig.cs.man.ac.uk
 mail  - Advanced Interfaces Group, Room 2.94, Kilburn Building, 
         University of Manchester, Manchester, M13 9PL, UK
*/


#include "polprivate.h"
#include <sys/time.h>
#include <stdio.h>
#include <termios.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>

int polhemusBox= -1;
int polhemusNum= -1;

extern int longranger;
extern int distortion;


/* Read a given number of characters from the Polhemus box */

void polhemusRead(int num, char *buf)
{
  fd_set pf;
  int rv, totread=0;

  do {
    FD_ZERO(&pf);
    FD_SET(polhemusBox, &pf);
    select(200, &pf, NULL, NULL, NULL);
    rv= read(polhemusBox, &buf[totread], num-totread);
    totread+=rv;
  } while (totread!=num);
  
  buf[num]=0;
}



/* Write a buffer to the Polhemus box  */

void polhemusWrite(char *fbuf)
{
  char buf[1000];
  int rv;

  /* Take a copy of the buffer */
  strcpy(buf, fbuf);

  /* A fullstop at the end signifies a newline  */
  if (buf[strlen(buf)-1]=='.') buf[strlen(buf)-1]=13;

  /* Write the buffer and check its all gone */
  rv= write(polhemusBox, buf, strlen(buf));
  if (rv!=strlen(buf)) printf("rv!=strlen(buf)\n");
}



/* Ensure nothing is waiting to be read */

void polhemusWait(void)
{
  char buf[1000];
  fd_set pf;
  struct timeval t;

  FD_ZERO(&pf);
  FD_SET(polhemusBox, &pf);

  /* A one second timeout */
  t.tv_sec=1;
  t.tv_usec=0;

  /* Read from the Polhemus box until there is nothing left */
  while (select(200, &pf, NULL, NULL, &t)) {
    FD_ZERO(&pf);
    FD_SET(polhemusBox, &pf);
    read(polhemusBox, buf, 1000);
  }
}



/* Sync the Polhemus box - read until we hit a newline */

void polhemusSync(void)
{
  char buf[1000];
  fd_set pf;

  FD_ZERO(&pf);
  FD_SET(polhemusBox, &pf);

  while (select(200, &pf, NULL, NULL, NULL)) {
    FD_ZERO(&pf);
    FD_SET(polhemusBox, &pf);
    read(polhemusBox, buf, 1);
    if (buf[0]==10) return;
  }
}



/* Ask for the number of receivers */

void polhemusNumr(void)
{
  char buf[1000];

  polhemusWrite("l1.");
  polhemusRead(9, buf);

  polhemusNum=0;
  polhemusNum+=buf[3]-48;
  polhemusNum+=buf[4]-48;
  polhemusNum+=buf[5]-48;
  polhemusNum+=buf[6]-48;

  if (polhemusNum<0 || polhemusNum>5) {
    printf("buf [%s]\n", buf);
    printf("errr, just trying that again\n");
    polhemusNumr();
  }
}



/* Open a connection with the Polhemus box */

int polhemusOpen(char *name, int poliso)
{
  struct termios mysettings;
  
  /* Open the serial port to the Polhemus box */
  polhemusBox = open(name, O_RDWR);

  if (polhemusBox == -1) {
    fprintf(stderr, "Could not open Polhemus box on device %s : ", name);
    perror("");
    return(0);
  }

  /* Read the present state of the port */
  if (tcgetattr(polhemusBox, &mysettings) == -1) {
    perror("Could not read Polhemus ioctl settings ");
    return(0);
  }

  /* Update it with our settings */
  mysettings.c_iflag= IGNBRK|IGNPAR;
  mysettings.c_cc[CEOF]= 1;
  mysettings.c_lflag= 0;
#ifdef TDM_IRIX5
  cfsetispeed(&mysettings, B38400); /* Cant you go any faster! */
#else
  cfsetispeed(&mysettings, B115200);
  cfsetospeed(&mysettings, B115200);
#endif

  /* Now set those settings onto the port */
  if (tcsetattr(polhemusBox, TCSANOW, &mysettings) == -1) {
    perror("Could not set Polhemus ioctl settings ");
    return(0);
  }

  /* Configure the Polhemus box */
  polhemusWrite("c");
  printf("Sync output...\n");
  polhemusWait();
  polhemusWrite("X.");
  sleep(1);
  polhemusSync();
  if (poliso)
  {
    polhemusNum= poliso;
  }
  else
  {
    printf("Asking for number of receivers...\n");
    polhemusNumr();
  }
  printf("%i receivers\n", polhemusNum);

  /* Set transmitter hemisphere */
  if (longranger) 
  {
    polhemusWrite("H1,1,0,0.");
    polhemusWrite("H2,1,0,0.");
    if (!poliso) {
      polhemusWrite("H3,1,0,0.");
      polhemusWrite("H4,1,0,0.");
    }
  }
  else
  {
    polhemusWrite("H1,0,0,-1.");
    polhemusWrite("H2,0,0,-1.");
    if (!poliso) {
      polhemusWrite("H3,0,0,-1.");
      polhemusWrite("H4,0,0,-1.");
    }
  }

  /* Ask for position and direction vectors to be reported */
  if (poliso)
  {
    polhemusWrite("O2,5,6,7.");
  }
  else
  {
    polhemusWrite("O1,2,5,6,7.");
    polhemusWrite("O2,2,5,6,7.");
    polhemusWrite("O3,2,5,6,7.");
    polhemusWrite("O4,2,5,6,7.");
  }

  printf("All done - switching to continuous mode...\n");
  polhemusWrite("C");

  return 1;
}



/* Fancy version of atof */

float myatof(char *buf, int st)
{
  char buf2[9];
  int i;

  for (i=0; i<7; i++) buf2[i]=buf[st+i];
  buf2[7]=0;

  return atof(buf2);
}



/* Endlessly read the Polhemus box */

void polhemusLoop(int debug)
{
  int i, lc=0;
  char buf2[1000];
  float xp, yp, zp, xx, xy, xz, yx, yy, yz, zx, zy, zz;
  struct timeval st, ed;
  DistortionGrid grid;

  gettimeofday(&st, NULL);

  /* read a distortion grid? */
  if (distortion) {
    if (longranger) {
      if (polhemusLoadDistortionGrid(&grid,"longranger.dat") == 0) {
	printf("can't read distortion grid\n");
	distortion= 0;
      }
    }
    else {
      printf("distortion corection only supported for longranger\n");
      distortion= 0;
    }
  }

  while (1) {
    lc++;

    for (i=0; i<polhemusNum; i++) {

      /* Read 87 bytes of data - one trackers worth */
      polhemusRead(87, buf2);

      /* Decode buffer into position */
      xp= myatof(buf2, 3);
      yp= myatof(buf2, 10);
      zp= myatof(buf2, 17);
	
      /* And direction vector */
      xx= myatof(buf2, 24);
      xy= myatof(buf2, 31);
      xz= myatof(buf2, 38);

      yx= myatof(buf2, 45);
      yy= myatof(buf2, 52);
      yz= myatof(buf2, 59);

      zx= myatof(buf2, 66);
      zy= myatof(buf2, 73);
      zz= myatof(buf2, 80);

      /* Store the position and local cordinate axis in the shared memory structure. */
      /* Have do to some site specific swapping of axis to account for our transmitter */
      /* and reciever orientation setup */
      if (longranger)
      {
	shared->pos[i].xpos = -yp;
	shared->pos[i].ypos = -zp;
	shared->pos[i].zpos = xp;
      }
      else
      {
	shared->pos[i].xpos = -xp;
	shared->pos[i].ypos = yp;
	shared->pos[i].zpos = -zp;
      }

      if (i==0 || i==1) 
      {
	if (longranger)
	{
	  shared->pos[i].u[0] = yz;  /* u=-z */
	  shared->pos[i].u[1] = zz;
	  shared->pos[i].u[2] = -xz;

	  shared->pos[i].v[0] = -yx; /* v=x */
	  shared->pos[i].v[1] = -zx;
	  shared->pos[i].v[2] = xx;
	
	  shared->pos[i].n[0] = yy;  /* n=-y */
	  shared->pos[i].n[1] = zy;
	  shared->pos[i].n[2] = -xy;
	}
	else
	{
	  shared->pos[i].u[0] = xz;  /* u=-z */
	  shared->pos[i].u[1] = -yz;
	  shared->pos[i].u[2] = zz;
 
	  shared->pos[i].v[0] = -xx; /* v=x */
	  shared->pos[i].v[1] = yx;
	  shared->pos[i].v[2] = -zx;
       
	  shared->pos[i].n[0] = xy;  /* n=-y */
	  shared->pos[i].n[1] = -yy;
	  shared->pos[i].n[2] = zy;
	}
      }
      else
      {
	if (longranger)
	{
	  shared->pos[i].u[0] = -yy; /* u=y */
	  shared->pos[i].u[1] = -zy;
	  shared->pos[i].u[2] = xy;

	  shared->pos[i].v[0] = yz;  /* v=-z */
	  shared->pos[i].v[1] = zz;
	  shared->pos[i].v[2] = -xz;
	    
	  shared->pos[i].n[0] = yx; /* n=-x */
	  shared->pos[i].n[1] = zx; 
	  shared->pos[i].n[2] = -xx;
	}
	else
	{
	  shared->pos[i].u[0] = -xy; /* u=y */
	  shared->pos[i].u[1] = yy;
	  shared->pos[i].u[2] = -zy;

	  shared->pos[i].v[0] = xz;  /* v=-z */
	  shared->pos[i].v[1] = -yz;
	  shared->pos[i].v[2] = zz;
	    
	  shared->pos[i].n[0] = xx; /* n=-x */
	  shared->pos[i].n[1] = -yx; 
	  shared->pos[i].n[2] = zx;
	}
      }

      /* correct for distortion? */
      shared->pos[i].status=0;
      if (distortion) {
	shared->pos[i].status |= TDM_CALIBRATED;
	if (!polhemusRemoveDistortion(&grid,i,shared->pos[i],&shared->pos[i])) 
	  shared->pos[i].status |= TDM_OUT_OF_RANGE;	  
      }

    }
    
    /* Print out record if debugging set */
    if (lc==100 && debug) {
      gettimeofday(&ed, NULL);
      printf("time %f\n", lc*1.0/((ed.tv_sec-st.tv_sec)+(ed.tv_usec-st.tv_usec)/1.0E+6));
      printf("[%s]\n", buf2);
      lc=0;
      st=ed;
    }
  }
}



/* Routine to close the Polhemus box */

void polhemusClose(void)
{
  printf("Closing Polhemus box\n");

  close(polhemusBox);
}
