/*
 TDM - various "Three D Mice" implementations for GNU Maverik
 Copyright (C) 1999-2001  Advanced Interfaces Group

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA


 The authors can be contacted via:
 www   - http://aig.cs.man.ac.uk
 email - tdm@aig.cs.man.ac.uk
 mail  - Advanced Interfaces Group, Room 2.94, Kilburn Building, 
         University of Manchester, Manchester, M13 9PL, UK
*/


#include "tdm.h"

#define SHM_KEY 1281 
#define MAX_NO_EVENTS  20

/* Polhemus box routines */

int  polhemusOpen(char *, int);
void polhemusLoop(int);
void polhemusClose(void);

/* Division box routines */

int  divisionOpen(char *);
void divisionLoop(int);
void divisionClose(void);

/* The shared memory data structure */

typedef struct {
  int event_in;                                      /* Array index to store the next button events */
  int event_out;                                     /* Array index of the most outstanding button events */
  TDM_buttonEvent event[MAX_NO_EVENTS];              /* The button event queue */ 
  int new_status[TDM_MAX_TRACKERS][TDM_MAX_BUTTONS]; /* The current button status */
  TDM_position pos[TDM_MAX_TRACKERS];                /* The current tracker position */
} Polhemus_shared;

extern Polhemus_shared *shared;

/* distortion grid */

typedef struct {
  float sx,sy,sz;
  float ex,ey,ez;
  float dx,dy,dz;
  int nx,ny,nz;
  unsigned char *valid;
  float *position;
  float *orientation;
  TDM_position last[TDM_MAX_TRACKERS];
} DistortionGrid;

int polhemusLoadDistortionGrid(DistortionGrid *, char *);
int polhemusRemoveDistortion(DistortionGrid *, int tracker, TDM_position, TDM_position *);


