/*
 TDM - various "Three D Mice" implementations for GNU Maverik
 Copyright (C) 1999-2001  Advanced Interfaces Group

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA


 The authors can be contacted via:
 www   - http://aig.cs.man.ac.uk
 email - tdm@aig.cs.man.ac.uk
 mail  - Advanced Interfaces Group, Room 2.94, Kilburn Building, 
         University of Manchester, Manchester, M13 9PL, UK
*/


#include "tdm.h"
#include "maverik.h"
#include <stdio.h>

#include <GL/glx.h>
#include <X11/Xlib.h>
#include <X11/keysym.h>
#include <X11/Xutil.h>
#include <X11/X.h>
#include "spwdata.h"
#include "spwinput.h"

TDM_position pos[TDM_MAX_TRACKERS];



/* Messy interface to expose Maverik's X state and event loop - need redoing in a cleaner manner */

typedef struct {
  char *name;
  Display *dpy;
  Window rootwin;
  int scrnum;
} MAVLIB_dpyhand;

typedef struct {
  int dpy;
  Window win;
  Window gfxwin;
  GLXContext ctx;
  int quad;
  int quad_separate;
} MAVLIB_winhand;

extern MAVLIB_dpyhand mavlib_dpy[];
extern MAVLIB_winhand mavlib_winhand[];
extern int mavlib_currwin;
extern int (*mavlib_extraXEventHandler)(XEvent);



/* Routine to be called when Maverik receives an unknown X event */

SPW_InputDispatchStruct *sbDispatch;
SPW_InputEvent *sbEvent;

int myfunc(XEvent e)
{
  XEvent n;

  /* Is it a SpaceBall event? */ 
  if (SPW_InputIsSpaceballEvent(mavlib_dpy[0].dpy, &e, sbEvent) == SPW_TRUE) 
  {
    /* If so deal with it */
    SPW_InputDispatchEx(sbDispatch);
    
    /* Any more event in the queue */
    if (XEventsQueued(mavlib_dpy[0].dpy, QueuedAfterReading)) {
      
      /* Is so take a peek and see if its a SpaceBall event */
      XPeekEvent(mavlib_dpy[0].dpy, &n);

      /* If it is deal with this one as well */
      if (SPW_InputIsSpaceballEvent(mavlib_dpy[0].dpy, &n, sbEvent) == SPW_TRUE) {
	XNextEvent(mavlib_dpy[0].dpy, &n);
	myfunc(n);
      }
    }
  }
  
  /* Return an arbitrary TRUE value greater than 6 to distinguish it from known X events */
  return 21;
}



/* Routines to be called by the SpaceBall event dispatcher */

void SbButtonPressEvent(Display *display, int button, void *appdata)
{
  printf("pressed %i\n", button);
}

void SbButtonReleaseEvent(Display *display, int button, void *pclientdata)
{
  printf("released %i\n", button);
}

void SbMotionEvent(Display *display, float *data, void *appdata)
{
  /* Update tracker 0's position. Ignore orientation for now */
  pos[0].xpos= data[0];
  pos[0].ypos= data[1];
  pos[0].zpos= -data[2]+20; /* Need to negate Z since pushing the ball away from the user 
			       increases Z - its a left handed system. Also add 20 to 
			       compensate for Maverik's default origin */
}



/* Routine to initialise the SpaceBall and set up event dispatcher structure */

int sp_init(void)
{
  int rv=1;
  
  /* Malloc off room for events */
  XEvent *event= mav_malloc(sizeof(XEvent)); /* X event */
  sbEvent= mav_malloc(sizeof(SPW_InputEvent)); /* Spaceball event */
  sbDispatch= mav_malloc(sizeof(SPW_InputDispatchStruct));  /* Spaceball dispatch structure */
   
  /* Check for SpaceBall */
  if (SPW_InputCheckForSpaceball(mavlib_dpy[0].dpy, mavlib_winhand[mavlib_currwin].win, "name") == SPW_FALSE) {
    fprintf(stderr, "Sorry - No Spaceball found.\n");
    rv=0;
  }

  /* Initialize Spaceball data struct with points to data values and callback functions */
  SPW_InputInitDispatchStruct(sbDispatch);
  sbDispatch->display= mavlib_dpy[0].dpy;
  sbDispatch->xevent= event;
  sbDispatch->spwevent= sbEvent;
  sbDispatch->handle_motion= SbMotionEvent;
  sbDispatch->handle_bpress= SbButtonPressEvent;
  sbDispatch->handle_brelease= SbButtonReleaseEvent;

  return rv;
}



/* Routine to initialize a trackers position */

void initTrk(TDM_position *pos)
{
  /* Position */
  pos->xpos=0;
  pos->ypos=0;
  pos->zpos=20;

  /* Trackers axis in transmitter frame */
  pos->u[0]=1; /* X axis */
  pos->u[1]=0;
  pos->u[2]=0;

  pos->v[0]=0; /* Y axis */
  pos->v[1]=1;
  pos->v[2]=0;

  pos->n[0]=0; /* Z axis */
  pos->n[1]=0;
  pos->n[2]=1;

  pos->status= 0;
}



/* Routine to initialize the trackers */

int TDM_init(void)
{
  int i;

  /* Initialise trackers */
  for (i=0; i<TDM_MAX_TRACKERS; i++) initTrk(&pos[i]);

  /* Set a function to be called when Maverik receives an unknown X event */
  mavlib_extraXEventHandler= myfunc;

  /* Init the SpaceBall */
  return sp_init();
}



/* Routine to get a single tracker's position */

void TDM_getPosition(int tracker, TDM_position *trk)
{
  /* Return the tracker's position and orientation */
  *trk= pos[tracker];
}



/* Routine to get the status (pressed/released) of the buttons */

void TDM_getButtons(int button_status[TDM_MAX_TRACKERS][TDM_MAX_BUTTONS])
{
}



/* Routine to get any outstanding button events */

int TDM_getEvent(TDM_buttonEvent *event)
{
  /* Return FALSE for no event */
  return 0;
}
