/*
 TDM - various "Three D Mice" implementations for GNU Maverik
 Copyright (C) 1999-2001  Advanced Interfaces Group

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA


 The authors can be contacted via:
 www   - http://aig.cs.man.ac.uk
 email - tdm@aig.cs.man.ac.uk
 mail  - Advanced Interfaces Group, Room 2.94, Kilburn Building, 
         University of Manchester, Manchester, M13 9PL, UK
*/


#include "so360private.h"
#include <stdio.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <signal.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#ifdef TDM_LINUX
#define SPACEORB_DEV "/dev/ttyS0"
#else
#ifdef __CYGWIN__
#define SPACEORB_DEV "/dev/COM1"
#else
#define SPACEORB_DEV "/dev/ttyd1"
#endif
#endif



/* The shared memory, and its ID */

so360_shared *shared;
int ShmID;



/* Routine to clean things up on exit */

void so360_atExit()
{
  /* Close comms */
  SpaceOrbClose();

  /* Clean up shared memory */
  shmctl(ShmID, IPC_RMID, NULL); 

  exit(0);
}



/* Main */

int main(int argc,char *argv[])
{
  int i, j=0, dbg=0;
  extern char *optarg;  
#ifdef TDM_LINUX
  FILE *f;
#endif
  char port[256];

  strcpy(port, SPACEORB_DEV);

  while ((i=getopt(argc,argv,"dhp:"))!=-1)
  {
    switch (i)
    {
    case 'd':
      dbg=1;
      break;

    case 'p':
      strcpy(port, optarg);
      break;

    case 'h':
    case '?':
      j++;
    }
  }

  if (j)
  {
    printf("usage: %s [-d] [-p so360 port]\n",argv[0]);
    exit(1); 
  }

  /* Clean up shared memory */
#ifdef TDM_LINUX
  system("ipcs -m | grep -i 504 | awk '{print $2}' > /tmp/.tmp");
  f=fopen("/tmp/.tmp","r");
  if (!f)
  {
    printf("Cant open /tmp/.tmp\n");
    exit(1);
  }
  
  if (fscanf(f,"%i",&ShmID)==1)
  {
    char buf[100];
    sprintf(buf, "ipcrm shm %i", ShmID);
    printf("exec [%s]\n", buf);
    system(buf);
  }
  system("rm -f /tmp/.tmp");
  
  fclose(f);
#else
  system("ipcrm -M 0x504 2>/dev/null"); /* 504= SHM_KEY in hex */
#endif

  /* Create the shared memory segment */
  ShmID=shmget(SHM_KEY,sizeof(so360_shared),(IPC_CREAT|IPC_EXCL|0666));

  /* Attach a pointer to it */ 
  shared=(so360_shared *)shmat(ShmID,0,0);

  if ((int)shared==-1)
  {
    perror("SpaceOrb360 daemon: Could not attach shared memory ");
    exit(-1);
  }

  /* Parent initalise shared variables */
  shared->event_in=0;
  shared->event_out=0;
  for (i=0; i<TDM_MAX_TRACKERS; i++)
  {
    shared->pos[i].xpos=0;
    shared->pos[i].ypos=0;
    shared->pos[i].zpos=20;
    shared->pos[i].u[0]=1;
    shared->pos[i].u[1]=0;
    shared->pos[i].u[2]=0;
    shared->pos[i].v[0]=0;
    shared->pos[i].v[1]=1;
    shared->pos[i].v[2]=0;
    shared->pos[i].n[0]=0;
    shared->pos[i].n[1]=0;
    shared->pos[i].n[2]=1;
    for (j=0; j<TDM_MAX_BUTTONS; j++)
      shared->new_status[i][j]=0;
  }

  /* Catch signals to remove shared memory  */
  signal(SIGHUP, so360_atExit);
  signal(SIGINT, so360_atExit);
  signal(SIGTERM, so360_atExit);

  /* Open connect with device enter an endless sampling loop */
  if (!SpaceOrbOpen(port)) so360_atExit();

  SpaceOrbLoop(dbg);

  so360_atExit();

  return 1;
}

