/*
 TDM - various "Three D Mice" implementations for GNU Maverik
 Copyright (C) 1999-2001  Advanced Interfaces Group

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA


 The authors can be contacted via:
 www   - http://aig.cs.man.ac.uk
 email - tdm@aig.cs.man.ac.uk
 mail  - Advanced Interfaces Group, Room 2.94, Kilburn Building, 
         University of Manchester, Manchester, M13 9PL, UK
*/


#include "mouseprivate.h"
#include <stdio.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <signal.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#ifdef TDM_LINUX
#define MOUSE_PORT "/dev/ttyS0"
#else
#ifdef __CYGWIN__
#define MOUSE_PORT "/dev/COM1"
#else
#define MOUSE_PORT "/dev/ttyd2"
#endif
#endif



/* The shared memory and its ID */

mouse_shared *shared;
int ShmID;



/* Routine to clean things up on exit */

void mouse_atExit()
{
  /* Close comms */
  mouseClose();

  /* Clean up shared memory */
  shmctl(ShmID, IPC_RMID, NULL); 

  exit(0);
}



/* Main */

int main(int argc, char *argv[])
{
#ifdef TDM_LINUX
  FILE *f;
#endif
  int i, j=0, dbg=0;
  extern char *optarg;
  char port[256];

  strcpy(port, MOUSE_PORT);
  while ((i=getopt(argc, argv, "dhp:"))!=-1) {
    switch (i) {
    case 'd':
      dbg=1;
      break;

    case 'p':
      strcpy(port, optarg);
      break;

    case 'h':
    case '?':
      j++;
    }
  }

  if (j) {
    printf("usage: %s [-d] [-p mouse port]\n", argv[0]);
    exit(1);
  }
  
  /* Clean up shared memory */
#ifdef TDM_LINUX
  system("ipcs -m | grep -i 505 | awk '{print $2}' > /tmp/.tmp");
  f= fopen("/tmp/.tmp", "r");
  if (!f) {
    printf("Cant open /tmp/.tmp\n");
    exit(1);
  }
  
  if (fscanf(f, "%i", &ShmID)==1) {
    char buf[100];
    sprintf(buf, "ipcrm shm %i", ShmID);
    printf("exec [%s]\n", buf);
    system(buf);
  }
  system("rm -f /tmp/.tmp");
    
  fclose(f);
#else
  system("ipcrm -M 0x505 2>/dev/null"); /* 505= SHM_KEY in hex */
#endif

  /* Create the shared memory segment */
  ShmID = shmget(SHM_KEY, sizeof(mouse_shared), (IPC_CREAT | IPC_EXCL | 0666));

  /* Attach a pointer to it */ 
  shared = (mouse_shared *) shmat(ShmID,0,0);

  if ((int)shared == -1) {
    perror("Serial mouse server: Could not attach shared memory ");
    exit(-1);
  }

  /* Initalise shared variables */
  shared->x= 50000;
  shared->y= 50000;
  shared->event_in= 0;
  shared->event_out= 0;
  shared->but[0]=0;
  shared->but[1]=0;
  shared->u[0]= 1;
  shared->u[1]= 0;
  shared->u[2]= 0;
  shared->v[0]= 0;
  shared->v[1]= 1;
  shared->v[2]= 0;
  shared->n[0]= 0;
  shared->n[1]= 0;
  shared->n[2]= 1;

  /* Catch signals to remove shared memory  */
  signal(SIGHUP, mouse_atExit);
  signal(SIGINT, mouse_atExit);
  signal(SIGTERM, mouse_atExit);
  
  /* Open connect with serial port and enter an endless sampling loop */
  if (!mouseOpen(port)) mouse_atExit();
  mouseLoop(dbg);

  mouse_atExit();

  return 1;
}
