/*
 TDM - various "Three D Mice" implementations for GNU Maverik
 Copyright (C) 1999-2001  Advanced Interfaces Group

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA


 The authors can be contacted via:
 www   - http://aig.cs.man.ac.uk
 email - tdm@aig.cs.man.ac.uk
 mail  - Advanced Interfaces Group, Room 2.94, Kilburn Building, 
         University of Manchester, Manchester, M13 9PL, UK
*/


#include "tdm.h"
#include <stdio.h>
#include <math.h>

int verbose=0;
TDM_position pos[TDM_MAX_TRACKERS];
float ang[TDM_MAX_TRACKERS];



/* Routine to initialize a trackers position */

void initTrk(TDM_position *pos)
{
  /* Position */
  pos->xpos=0;
  pos->ypos=0;
  pos->zpos=20;

  /* Trackers axis in transmitter frame */
  pos->u[0]=1; /* X axis */
  pos->u[1]=0;
  pos->u[2]=0;

  pos->v[0]=0; /* Y axis */
  pos->v[1]=1;
  pos->v[2]=0;

  pos->n[0]=0; /* Z axis */
  pos->n[1]=0;
  pos->n[2]=1;

  pos->status= 0;
}



/* Routine to initialize the trackers */

int TDM_init(void)
{
  int i;

  if (verbose) printf("initialise trackers\n");

  /* Initialise trackers */
  for (i=0; i<TDM_MAX_TRACKERS; i++) {
    initTrk(&pos[i]);
    ang[i]=0;
  }

  /* Return TRUE for successful initialization */
  return 1;
}



/* Routine to get a single tracker's position */

void TDM_getPosition(int tracker, TDM_position *trk)
{
  if (verbose) printf("get position of tracker %i\n", tracker);

  /* Wiggle the trackers */
  if (tracker==0) {
    pos[tracker].xpos= 10*sin(ang[tracker]); 
    pos[tracker].ypos= 10*cos(ang[tracker]); 
    ang[tracker]+=0.01;
  }

  if (tracker==1) {
    pos[tracker].xpos= -10*sin(ang[tracker]); 
    ang[tracker]+=0.03;
  }

  if (tracker==2) {
    pos[tracker].ypos= -10*sin(ang[tracker]); 
    ang[tracker]+=0.02;
  }

  if (tracker==3) {
    pos[tracker].zpos= 15+10*sin(ang[tracker]); 
    ang[tracker]+=0.02;
  }

  /* Return the tracker's position and orientation */
  *trk= pos[tracker];
}



/* Routine to get the status (pressed/released) of the buttons */

void TDM_getButtons(int button_status[TDM_MAX_TRACKERS][TDM_MAX_BUTTONS])
{
  if (verbose) printf("get buttons\n");
}



/* Routine to get any outstanding button events */

int TDM_getEvent(TDM_buttonEvent *event)
{
  if (verbose) printf("checking for any events\n");

  /* Return FALSE for no event */
  return 0;
}
